/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"

#include "Client.h"
#include "ConfigurationDialog.h"
#include "CppUtil.h"
#include "Debug.h"
#include "MainWindow.h"

#include <QApplication>

//___________________________________________
Application::Application( CommandLineArguments arguments ):
    BaseApplication( 0, arguments ),
    Counter( "Application" )
{}

//___________________________________________
bool Application::realizeWidget()
{

    Debug::Throw( "Application::RealizeWidget.\n" );
    if( !BaseApplication::realizeWidget() ) return false;

    mainWindow_ = new MainWindow( nullptr );

    connect( mainWindow_,
        SIGNAL(requestCommand(Server::ServerCommand::CommandType)),
        SLOT(sendServerCommand(Server::ServerCommand::CommandType)) );

    connect( mainWindow_,
        SIGNAL(requestCommand(Server::ServerCommand)),
        SLOT(sendServerCommand(Server::ServerCommand)) );

    connect(
        &_applicationManager(), SIGNAL(initialized()),
        &mainWindow_->resetAction(), SLOT(trigger()) );

    // update configuration
    emit configurationChanged();

    // send first request (to identify all applications)
    _requestIdentify();

    // show main widget
    mainWindow_->centerOnDesktop();
    mainWindow_->show();

    return true;

}

//___________________________________________
bool Application::initApplicationManager()
{

    Debug::Throw( "Application::initApplicationManager.\n" );

    // make sure application manager does not already exist
    if( _hasApplicationManager() ) return true;

    // parse arguments and forbid --no-server options
    CommandLineParser parser( commandLineParser( _arguments() ) );
    if( parser.hasFlag( "--no-server" ) )
    {

        Debug::Throw(0) << "serverlog - by definition the application cannot work in 'no-server' mode." << endl;
        Debug::Throw(0) << "Aborting." << endl;
        return false;

    } else if( !BaseApplication::initApplicationManager() ) {

        return false;

    }

    // make sure server-client is connected
    _connectClient();
    connect( &_applicationManager(), SIGNAL(initialized()), SLOT(_connectClient()) );

    return true;
}


//____________________________________________
void Application::usage() const
{
    _usage( "serverlog" );
    commandLineParser().usage();
    return;
}

//_______________________________________________
void Application::_configuration()
{

    Debug::Throw( "Application::_configuration.\n" );

    // save configuration
    emit saveConfiguration();

    ConfigurationDialog dialog(0);
    connect( &dialog, SIGNAL(configurationChanged()), SIGNAL(configurationChanged()) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();

}

//___________________________________________
void Application::_connectClient()
{
    Debug::Throw( "Application::_connectClient.\n" );
    if( _hasApplicationManager() && _applicationManager().hasClient() )
    {
        connect(
            &_applicationManager().client(), SIGNAL(commandAvailable(Server::ServerCommand)),
            this, SLOT(_append(Server::ServerCommand)), Qt::UniqueConnection );
    }
}

//___________________________________________
void Application::_append( Server::ServerCommand command )
{

    Debug::Throw( "Application::_append.\n" );

    if( mainWindow_ )
    { mainWindow_->append( Base::makeT<Server::ServerCommand::List>({ command }) ); }

}

//________________________________________________
bool Application::_processCommand( Server::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == Server::ServerCommand::Raise )
    {
        if( mainWindow_ ) mainWindow_->uniconify();
        return true;
    } else return false;

}
