/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MainWindow.h"

#include "Application.h"
#include "BaseStatusBar.h"
#include "Color.h"
#include "ContextMenu.h"
#include "CustomToolBar.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "MenuBar.h"
#include "QtUtil.h"
#include "Singleton.h"
#include "TreeView.h"
#include "XmlOptions.h"

#include <QLayout>

//___________________________________________
MainWindow::MainWindow( QWidget* parent ):
    BaseMainWindow( parent )
{
    setOptionName( "MAIN_WINDOW" );
    Debug::Throw( "MainWindow::MainWindow.\n" );

    // install actions
    _installActions();
    auto application = Base::Singleton::get().application<Application>();
    addAction( &application->closeAction() );

    // main widget and layout
    QWidget* main = new QWidget( this );
    main->setLayout(  new QVBoxLayout );
    main->layout()->setMargin(0);
    main->layout()->setSpacing(5);
    setCentralWidget( main );

    // toolbar
    toolbar_ = new CustomToolBar( tr( "Main Toolbar" ), this, "TOOLBAR" );
    toolBar().addAction( &resetAction() );
    toolBar().addAction( &unlockAction() );
    toolBar().addAction( &raiseAction() );
    toolBar().addAction( &abortAction() );

    // command list
    treeView_ = new TreeView;
    main->layout()->addWidget( new TreeView::Container( main, treeView_ ) );
    treeView_->setModel( &model_ );
    treeView_->setSortingEnabled( true );
    treeView_->setRootIsDecorated( true );
    treeView_->setSelectionMode( TreeView::ContiguousSelection );
    treeView_->setOptionName( "LIST" );

    QMenu* menu( new ContextMenu( treeView_ ) );
    menu->addAction( &resetAction() );
    menu->addAction( &unlockAction() );
    menu->addSeparator();
    menu->addAction( &raiseAction() );
    menu->addAction( &abortAction() );
    menu->addSeparator();
    menu->addAction( &toolBar().visibilityAction() );

    connect( treeView_->selectionModel(), SIGNAL(selectionChanged (QItemSelection,QItemSelection)), SLOT(_updateActions()) );
    connect( treeView_, SIGNAL(activated(QModelIndex)), SLOT(_requestRaise()) );

    // status bar
    setStatusBar( statusBar_ = new BaseStatusBar( this ) );
    statusBar_->addLabel( 1 );
    statusBar_->addClock();

    // menu bar
    setMenuBar( new MenuBar( this ) );

    connect( application, SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();
    _updateActions();

}

//_______________________________________
void MainWindow::append( const ServerCommandModel::List& commands )
{

    Debug::Throw( "MainWindow::append.\n" );

    // update state frame
    ServerCommandModel::List addedCommands;
    ServerCommandModel::List removedCommands;
    for( const auto& command:commands )
    {

        statusBar_->label().setTextAndUpdate( tr( "Recieved %1 from %2" ).arg( command.commandName(), command.id().name() ) );

        if( verbosity_ == VerbosityLow && (
            command.command() == Server::ServerCommand::GeometryRequest ||
            command.command() == Server::ServerCommand::Geometry ||
            command.command() == Server::ServerCommand::Abort ||
            command.command() == Server::ServerCommand::Raise ||
            command.command() == Server::ServerCommand::Alive ||
            command.command() == Server::ServerCommand::Denied ) )
        { continue; }

        // for KILLED command and low verbosity, removes corresponding item
        if( verbosity_ == VerbosityLow && command.command() == Server::ServerCommand::Killed )
        {

            auto commands( model_.children() );
            for( const auto& other:commands )
            { if( other.id() == command.id() ) removedCommands.append( other ); }
            continue;

        }

        addedCommands.append( command );
    }

    model_.remove( removedCommands );
    model_.add( addedCommands );

    treeView_->resizeColumnToContents( ServerCommandModel::Name );

    return;
}

//____________________________________
void MainWindow::closeEvent( QCloseEvent *event )
{
    Debug::Throw( "MainWindow::closeEvent.\n" );
    event->accept();
}

//___________________________________________________________
void MainWindow::_updateConfiguration()
{

    Debug::Throw( "MainWindow::_updateConfiguration.\n" );

    // resize
    setVerbosity( (Verbosity) XmlOptions::get().get<int>( "VERBOSITY" ) );

}

//_______________________________________
void MainWindow::_updateActions()
{

    Debug::Throw( "MainWindow::_updateActions.\n" );
    bool hasSelection( !treeView_->selectionModel()->selectedIndexes().empty() );
    raiseAction().setEnabled( hasSelection );
    abortAction().setEnabled( hasSelection );
    return;
}

//________________________________________
void MainWindow::_requestReset()
{
    Debug::Throw( "MainWindow::_requestReset.\n" );
    statusBar_->label().setTextAndUpdate( tr( "Resetting list..." ) );
    model_.clear();
    emit requestCommand( Server::ServerCommand::Identify );
}

//________________________________________
void MainWindow::_requestUnlock()
{
    Debug::Throw( "MainWindow::_requestUnlock.\n" );
    statusBar_->label().setTextAndUpdate( tr( "Unlocking registered applications..." ) );
    model_.clear();
    emit requestCommand( Server::ServerCommand::Unlock );
    emit requestCommand( Server::ServerCommand::Identify );
}

//________________________________________
void MainWindow::_requestRaise()
{
    Debug::Throw( "MainWindow::_requestRaise.\n" );
    ServerCommandModel::List commands( model_.get( treeView_->selectionModel()->selectedRows() ) );
    for( const auto command:commands )
    { emit requestCommand( Server::ServerCommand( command.id(), Server::ServerCommand::Raise ) ); }
}

//________________________________________
void MainWindow::_requestAbort()
{
    Debug::Throw( "MainWindow::_requestAbort.\n" );

    ServerCommandModel::List commands( model_.get( treeView_->selectionModel()->selectedRows() ) );
    for( const auto& command:commands )
    { emit requestCommand( Server::ServerCommand( command.id(), Server::ServerCommand::Abort ) ); }
    _requestReset();

}

//________________________________________
void MainWindow::_installActions()
{
    Debug::Throw( "MainWindow::_installAction.\n" );

    addAction( resetAction_ = new QAction( IconEngine::get( IconNames::Reset ), tr( "Reload" ), this ) );
    resetAction_->setShortcut( QKeySequence::Refresh );
    connect( resetAction_, SIGNAL(triggered()), SLOT(_requestReset()) );

    addAction( unlockAction_ = new QAction( IconEngine::get( IconNames::Unlock ), tr( "Unlock" ), this ) );
    connect( unlockAction_, SIGNAL(triggered()), SLOT(_requestUnlock()) );

    // actions
    addAction( raiseAction_ = new QAction( IconEngine::get( IconNames::Find ), tr( "Raise" ), this ) );
    raiseAction_->setToolTip( tr( "Raise selected applications" ) );
    connect( raiseAction_, SIGNAL(triggered()), SLOT(_requestRaise()) );


    addAction( abortAction_ = new QAction( IconEngine::get( IconNames::Abort ), tr( "Abort" ), this ) );
    abortAction_->setToolTip( tr( "Abort selected applications" ) );
    connect( abortAction_, SIGNAL(triggered()), SLOT(_requestAbort()) );

}
