/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MenuBar.h"

#include "Application.h"
#include "CustomToolBar.h"
#include "DebugMenu.h"
#include "MainWindow.h"
#include "Singleton.h"
#include "XmlOptions.h"

//_______________________________________________
MenuBar::MenuBar( QWidget* parent ):
    QMenuBar( parent ),
    Counter( "MenuBar" )
{
    Debug::Throw( "MenuBar::MenuBar.\n" );

    // application
    auto application( Base::Singleton::get().application<Application>() );
    auto mainWindow( static_cast<MainWindow*>( window() ) );

    // generic menu
    QMenu* menu;

    // file menu
    menu = addMenu( tr( "File" ) );
    menu->addAction( &application->closeAction() );

    // create edit menu
    menu = addMenu( tr( "Tools" ) );
    menu->addAction( &mainWindow->resetAction() );
    menu->addAction( &mainWindow->unlockAction() );
    menu->addSeparator();
    menu->addAction( &mainWindow->raiseAction() );
    menu->addAction( &mainWindow->abortAction() );

    // Settings menu
    menu = addMenu( tr( "Settings" ) );

    // verbosity menu
    verbosityMenu_ = menu->addMenu( tr( "Verbosity" ) );
    QActionGroup* verbosity_group = new QActionGroup( menu );
    verbosity_group->setExclusive( true );
    verbosityLow_ = verbosity_group->addAction( tr( "Low" ) );
    verbosityHigh_ = verbosity_group->addAction( tr( "High" ) );
    verbosityLow_->setCheckable( true );
    verbosityHigh_->setCheckable( true );
    verbosityMenu_->addActions( verbosity_group->actions() );

    connect( verbosityMenu_, SIGNAL(aboutToShow()), SLOT(_updateVerbosityMenu()) );
    connect( verbosityMenu_, SIGNAL(triggered(QAction*)), SLOT(_toggleVerbosity(QAction*)) );

    menu->addSeparator();
    menu->addAction( &application->configurationAction() );

    // help menu
    menu = addMenu( tr( "Help" ) );
    menu->addAction( &application->aboutAction() );
    menu->addAction( &application->aboutQtAction() );

    // debug menu
    menu->addSeparator();
    DebugMenu *debugMenu( new DebugMenu( menu ) );
    menu->addAction( debugMenu->menuAction() );
    debugMenu->menuAction()->setVisible( false );

}

//_______________________________________________
void MenuBar::_updateVerbosityMenu()
{
    Debug::Throw( "MenuBar::_UpdateVerbosityMenu.\n" );

    const int verbosity( XmlOptions::get().get<int>("VERBOSITY") );
    verbosityLow_->setChecked( verbosity == MainWindow::VerbosityLow );
    verbosityHigh_->setChecked( verbosity == MainWindow::VerbosityHigh );

}

//_______________________________________________
void MenuBar::_toggleVerbosity( QAction* )
{
    Debug::Throw( "MenuBar::_toggleVerbosity.\n" );

    XmlOptions::get().set<int>( "VERBOSITY", (verbosityLow_->isChecked()) ? MainWindow::VerbosityLow : MainWindow::VerbosityHigh );
    static_cast<MainWindow*>(window())->setVerbosity( (MainWindow::Verbosity) XmlOptions::get().get<int>( "VERBOSITY" ) );

}
